import { world, system, ItemStack, Items, MinecraftEffectTypes, MinecraftEntityTypes, DynamicPropertiesDefinition } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";
import "./mobHeads.js";
import "./armorEquip.js";
import "./customOre.js";
import "./combatCalculations.js";
import "./food.js";
import "./prayer.js";


const blockMap = new Map();

/**
 * Register a tick timeout
 * @param {(data: BeforeChatEvent, args: Array<string>) => void} callback Code you want to execute when the command is executed
 * @param {number} tick time in ticks you want the return to occur
 * @example
 *  SA.TickTimeOutBuild.setTickTimeout(function () {
 *    console.log(`callback`)
 * });
 */
export function setTickTimeout(callback, tick) {
    new tickTimeout(callback, tick);
}
/**
 * Delay executing a function, REPEATEDLY
 * @param {(data: BeforeChatEvent, args: Array<string>) => void} callback Code you want to execute when the command is executed
 * @param {number} tick time in ticks you want the return to occur
 * @returns {number}
 */
export function setTickInterval(callback, tick) {
    new tickTimeout(callback, tick, true);
}
class tickTimeout {
    /**
     * Register a timeout
     * @param {(data: BeforeChatEvent, args: Array<string>) => void} callback Code you want to execute when the command is executed
     * @param {number} time time in Date want the function to run
     */
    constructor(callback, tick, loop = false) {
        this.tickDelay = tick;
        this.callbackTick = 0;
        this.loop = loop;

        let TickCallBack = world.events.tick.subscribe((data) => {
            if (this.callbackTick == 0) {
                this.callbackTick = data.currentTick + this.tickDelay;
            }
            try {
                if (this.callbackTick <= data.currentTick) {
                    // return callback
                    callback();
                    if (this.loop) {
                        this.callbackTick = data.currentTick + this.tickDelay;
                    } else {
                        world.events.tick.unsubscribe(TickCallBack);
                    }
                }
            } catch (error) {
                console.warn(`${error} : ${error.stack}`);
            }
        });
    }
}

//displays a welcome message to the player
/*world.events.playerJoin.subscribe(eventData => {
    let { player } = eventData

    let event = world.events.tick.subscribe(async () => {
        if (!player) {
            world.events.tick.unsubscribe(event)
            return
        }

        try {
            await sleep(100);
            player.runCommandAsync("tellraw @s {\"rawtext\": [{\"text\": \"\u00A7eYou are playing RuneCraft [0.7]\nDeveloped by Estrongel\" }]}")

            player = null
        } catch { }

    })

})*/

//gets the player score value
function getPlayerScore(scoreboardValue, player) {
    return world.scoreboard.getObjective(scoreboardValue).getScore(player.scoreboard)
}

//function to display scoreboard
function leaderboards(scoreboardValueToSearch, scoreboardNameToSearch, player) {
    let playerArray = [...world.getPlayers()];
    let playerTransfer = 0;
    let playerCount = 0;
    for (let i = 0; i < playerArray.length - 1; i++) {
        for (let x = 1; x < playerArray.length - 1; x++) {
            let p1 = playerArray[i];
            let p2 = playerArray[i + x];
            if (getPlayerScore(scoreboardValueToSearch, p1.scoreboard) <= getPlayerScore(scoreboardValueToSearch, p2.scoreboard)) {
                playerTransfer = playerArray[i];
                playerArray[i] = playerArray[i + x];
                playerArray[i + x] = playerArray[i];
            }
        }
        playerCount++;
    }

    for (let i = 0; i < 8 && i < playerArray.length; i++) {
        let playerIterator = playerArray[i];
        let playerRank = (i + 1);
        //let playerName = playerArray[i].name;
        //let scoreboardValue = getPlayerScore(scoreboardValueToSearch, playerIterator);
        //let scoreboardName = getPlayerScore(scoreboardNameToSearch, playerIterator);
        player.tell(`${playerRank}.${playerArray[i].name} Lv.${getPlayerScore(scoreboardNameToSearch, playerIterator)} ${getPlayerScore(scoreboardValueToSearch, playerIterator)}`);
    }
}

//function to get player scoreboards
//performs all of the custom chat commands
world.events.beforeChat.subscribe((chatEvent) => {
    let message = chatEvent.message
    if (message.startsWith('!')) {
        message.toLowerCase();
        chatEvent.cancel = true;
        switch (message) {
            //help list
            case "!about":
            case "!runecraft":
            case "!rc":
                chatEvent.sender.tell(`\u00A72==========About==========\n\u00A7rRuneCraft is an add-on heavily inspired by Runescape that seeks to bring the MMORPG experience to Minecraft survival worlds in the form of unique skills and interactive systems. The add-on is developed by Estrongel and receives occasional help from community members in its development.`)
                break;
            case "!help":
            case "!cmds":
            case "!h":
            case "!help 1":
            case "!cmds 1":
            case "!h1":
                chatEvent.sender.tell(`\u00A72==========Command List (1/2)==========\n\u00A7a<!about> \u00A7dDisplays RuneCraft info\n\u00A7a<!help (1-2)>\u00A7dDisplays a list of commands\n\u00A7a<!stats> \u00A7dDisplays skill levels\n\u00A7a<!skills> \u00A7dDisplays a list of skills\n\u00A7a<!teams> \u00A7dTeam info\n\u00A7a<!leaderboards> \u00A7dDisplays Leaderboard Info\n\u00A7a<!wiki> \u00A7dRuneCraft Wiki\n\u00A7a<!youtube> \u00A7dSupport me`)
                break;
            case "!help 2":
            case "!cmds 2":
            case "!h2":
                chatEvent.sender.tell(`\u00A72==========Command List (2/2)==========\n\u00A7a<!discord> \u00A7dRuneCraft Discord server`)
                break;
            //skill list
            case "!skills":
            case "!skill":
            case "!skil":
            case "!sk1":
                chatEvent.sender.tell(`\u00A72==========Skills (1/2)==========\n\u00A7a<!agility> \u00A7dDisplays Agility info\n\u00A7a<!attack> \u00A7dDisplays Attack info\n\u00A7a<!constitution> \u00A7dDisplays Constitution info\n\u00A7a<!defense> \u00A7dDisplays Defense info\n\u00A7a<!digging> \u00A7dDisplays Digging info\n\u00A7a<!farming> \u00A7dDisplays Farming info\n\u00A7a<!prayer> \u00A7dDisplays Prayer info\n\u00A7a<!ranged> \u00A7dDisplays Ranged info\n\u00A7a<!thieving> \u00A7dDisplays Thieving info`)
                break;
            case "!skills 2":
            case "!skills2":
            case "!sk2":
            case "!skil 2":
                chatEvent.sender.tell(`\u00A72==========Skills (2/2)==========\n\u00A7a<!mining> \u00A7dDisplays Mining info\n\u00A7a<!prayer> \u00A7dDisplays Prayer info\n\u00A7a<!ranged> \u00A7dDisplays Ranged info\n\u00A7a<!thieving> \u00A7dDisplays Thieving info\n\u00A7a<!Woodcutting> \u00A7dDisplays Woodcutting info`)
                break;
            //leaderboards
            case "!leaderboard":
            case "!leaderboards":
            case "!lb":
                chatEvent.sender.tell(`\u00A72==========Leaderbords==========\n\u00A7a<!leaderboard total> \u00A7dDisplays Total Level Leaderboard\n\u00A7a<!leaderboard agility> \u00A7dDisplays Agility Leaderboard\n\u00A7a<!leaderboard attack> \u00A7dDisplays Attack Leaderboard\n\u00A7a<!leaderboard constitution> \u00A7dDisplays Constitution Leaderboard\n\u00A7a<!leaderboard defense> \u00A7dDisplays Defense Leaderboard\n\u00A7a<!leaderboard prayer> \u00A7dDisplays Prayer Leaderboard\n\u00A7a<!leaderboard ranged> \u00A7dDisplays Ranged Leaderboard\n\u00A7a<!leaderboard thieving> \u00A7dDisplays Thieving Leaderboard`)
                break;
            //agility leaderboard
            case "!lba":
            case "!leaderboard agility":
                let sender = chatEvent.sender;
                leaderboards("tot_agility_xp", "agility_level", sender);
                break;
            //agility skill pages
            case "!agility":
            case "!ag":
            case "!agi":
                chatEvent.sender.tell(`\u00A72==========Agility==========\n\u00A7rAgility is a non-combat skill trained through traversing the world. Leveling up this skill provides bonuses to stamina regeneration. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!agility unlocks> \u00A7dDisplays Agility unlock info`)
                break;
            //agility unlocks
            case "!agility unlocks":
            case "!agility unlock":
            case "!agu":
                chatEvent.sender.tell(`\u00A72==========Agility Unlocks==========\n\u00A7aScaling Passive: \u00A7eEvery \u00A769\u00A7e levels gain 0.1% increased stamina regeneration up to level \u00A7699\u00A7e\n\u00A7aLvl.\u00A761 \u00A7rWear the full\u00A7b Graceful Set\n\u00A7aLvl.\u00A7699 \u00A7rWear the\u00A7b Agility Skillcape`)
                break;
            //attack
            case "!attack":
            case "!att":
            case "!at":
                chatEvent.sender.tell(`\u00A72==========Attack==========\n\u00A7rAttack is a combat skill trained through fighting enemies with melee weapons. Leveling up this skill will grant you the ability to wield more powerful weapons. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!attack unlocks> \u00A7dDisplays Attack unlock info`)
                break;
            //attack unlocks
            case "!attack unlocks":
            case "!attack unlock":
            case "!atu":
                chatEvent.sender.tell(`\u00A72==========Attack Unlocks==========\n\u00A7aLvl.\u00A761 \u00A7rEquip\u00A7b Wood Sword\n\u00A7aLvl.\u00A7610 \u00A7rEquip\u00A7b Bronze Sword\n\u00A7aLvl.\u00A7615 \u00A7rEquip\u00A7b Iron Sword\n\u00A7aLvl.\u00A7625 \u00A7rEquip\u00A7b Black Sword\n\u00A7aLvl.\u00A7630 \u00A7rEquip\u00A7b Steel Sword\n\u00A7aLvl.\u00A7640 \u00A7rEquip\u00A7b Mithril Sword\n\u00A7aLvl.\u00A7645 \u00A7rEquip\u00A7b Adamant Sword\n\u00A7aLvl.\u00A7655 \u00A7rEquip\u00A7b Rune Sword`)
                break;
            //constitution
            case "!constitution":
            case "!const":
            case "!con":
                chatEvent.sender.tell(`\u00A72==========Constitution==========\n\u00A7rConstitution is a skill trained through fighting enemies. Leveling up this skill will increase your health at specific intervals. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!constitution unlocks> \u00A7dDisplays Constitution unlock info`)
                break;
            case "!constitution unlocks":
            case "!const unlocks":
            case "!con unlocks":
            case "!conu":
                chatEvent.sender.tell(`\u00A72==========Constitution Unlocks==========\n\u00A7aScaling Passive: \u00A7eStarting at level \u00A764\u00A7e, and every \u00A765\u00A7e levels, gain 1 Max Health up to level \u00A7699\u00A7e\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7bConstitution Skillcape`)
                break;
            //defense
            case "!defense":
            case "!defence":
            case "!def":
                chatEvent.sender.tell(`\u00A72==========Defense==========\n\u00A7rDefense is a skill trained through fighting enemies. Leveling up this skill will grant you the ability to wear more more powerful equipment. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!defense unlocks> \u00A7dDisplays Defense unlock info`)
                break;
            //prayer
            case "!prayer":
            case "!pray":
            case "!pr":
                chatEvent.sender.tell(`\u00A72==========Prayer==========\n\u00A7rPrayer is a semi-combat skill that grants you powerful buffs. Leveling up this skill will grant you the ability to activate stronger prayers over a longer period of time. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!prayer unlocks (1-3)> \u00A7dDisplays Prayer unlock info\n\u00A7a<!prayer regeneration> \u00A7dDisplays Prayer regeneration info`)
                break;
            case "!prayer regeneration":
            case "!prayer regen":
            case "!preg":
                chatEvent.sender.tell(`\u00A72==========Prayer Regeneration==========\n\u00A7rPrayer Points are not regenerated automatically like Stamina. In order to recover Prayer Points, players will need to interact with either a Gilded or Chaos Altar.`)
                break;
            case "!prayer unlocks 1":
            case "!pray unlocks 1":
            case "!pu1":
            case "!pu":
            case "!pu 1":
            case "!prayer unlocks":
            case "!pray unlocks":
                chatEvent.sender.tell(`\u00A72==========Prayer Unlocks (1/3)==========\n\u00A7aLvl.\u00A765 \u00A7rActivate\u00A7b Jump Boost 1\n\u00A7aLvl.\u00A769 \u00A7rActivate\u00A7b Stamina Regeneration 1\n\u00A7aLvl.\u00A7613 \u00A7rActivate\u00A7b Regeneration 1\n\u00A7aLvl.\u00A7617 \u00A7rActivate\u00A7b Speed 1\n\u00A7aLvl.\u00A7621 \u00A7rActivate\u00A7b Strength 1\n\u00A7aLvl.\u00A7625 \u00A7rActivate\u00A7b Resistance 1\n\u00A7aLvl.\u00A7629 \u00A7rActivate\u00A7b Levitation\n\u00A7aLvl.\u00A7633 \u00A7rActivate\u00A7b Slow Falling`)
                break;
            case "!prayer unlocks 2":
            case "!pray unlocks 2":
            case "!pu2":
            case "!pu 2":
                chatEvent.sender.tell(`\u00A72==========Prayer Unlocks (2/3)==========\n\u00A7aLvl.\u00A7637 \u00A7rActivate\u00A7b Water Breathing\n\u00A7aLvl.\u00A7641 \u00A7rActivate\u00A7b Invisibility\n\u00A7aLvl.\u00A7645 \u00A7rActivate\u00A7b Fire Resistance\n\u00A7aLvl.\u00A7649 \u00A7rActivate\u00A7b Jump Boost 2\n\u00A7aLvl.\u00A7653 \u00A7rActivate\u00A7b Stamina Regeneration 2\n\u00A7aLvl.\u00A7657 \u00A7rActivate\u00A7b Regeneration 2\n\u00A7aLvl.\u00A7661 \u00A7rActivate\u00A7b Speed 2\n\u00A7aLvl.\u00A7665 \u00A7rActivate\u00A7b Strength 2`)
                break;
            case "!prayer unlocks 3":
            case "!pray unlocks 3":
            case "!pu3":
            case "!pu 3":
                chatEvent.sender.tell(`\u00A72==========Prayer Unlocks (3/3)==========\n\u00A7aLvl.\u00A7669 \u00A7rActivate\u00A7b Resistance 2\n\u00A7aLvl.\u00A7673 \u00A7rActivate\u00A7b Haste\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7bPrayer Skillcape\n\u00A7aLvl.\u00A76120 \u00A7rUnlock the \u00A7bPrayer Skillcape Mastery \u00A7rtexture`)
                break;
            //thieving
            case "!thieving":
            case "!thiev":
            case "!t":
                chatEvent.sender.tell(`\u00A72==========Thieving==========\n\u00A7rThieving is a non-combat skill trained by stealing from mobs. Leveling up this skill will grant you the ability to steal from different mobs. For detailed skill infomration, check the RuneCraft Wiki. \n\u00A7a<!thieving unlocks (1-3)> \u00A7dDisplays Thieving unlock info`)
                break;
            //ranged
            case "!thieving unlocks 1":
            case "!thiev unlocks 1":
            case "!tu1":
            case "!tu 1":
            case "!thieving unlocks":
            case "!thiev unlocks":
            case "!tu":
                chatEvent.sender.tell(`\u00A72==========Thieving Unlocks (1/3)==========\n\u00A7aLvl.\u00A761 \u00A7rWear the full\u00A7b Rogue Set\n\u00A7aLvl.\u00A761 \u00A7rPickpocket\u00A79 Nitwits\n\u00A7aLvl.\u00A767 \u00A7rPickpocket\u00A79 Villager\n\u00A7aLvl.\u00A7613 \u00A7rPickpocket\u00A79 Fisherman\n\u00A7aLvl.\u00A7619 \u00A7rPickpocket\u00A79 Butcher\n\u00A7aLvl.\u00A7625 \u00A7rPickpocket\u00A79 Leatherworker\n\u00A7aLvl.\u00A7630 \u00A7rPickpocket\u00A79 Cartographer\n\u00A7aLvl.\u00A7635 \u00A7rPickpocket\u00A79 Shepherd`)
                break;
            case "!thieving unlocks 2":
            case "!thiev unlocks 2":
            case "!tu2":
            case "!tu 2":
                chatEvent.sender.tell(`\u00A72==========Thievings (2/3)==========\n\u00A7aLvl.\u00A7640 \u00A7rPickpocket\u00A79 Toolsmith\n\u00A7aLvl.\u00A7645 \u00A7rPickpocket\u00A79 Mason\n\u00A7aLvl.\u00A7649 \u00A7rPickpocket\u00A79 Fletcher\n\u00A7aLvl.\u00A7653 \u00A7rPickpocket\u00A79 Weaponsmith\n\u00A7aLvl.\u00A7657 \u00A7rPickpocket\u00A79 Farmer\n\u00A7aLvl.\u00A7661 \u00A7rPickpocket\u00A79 Wandering Trader\n\u00A7aLvl.\u00A7664 \u00A7rPickpocket\u00A79 Armorer\n\u00A7aLvl.\u00A7667 \u00A7rPickpocket\u00A79 Cleric`)
                break;
            case "!thieving unlocks 3":
            case "!thiev unlocks 3":
            case "!tu3":
            case "!tu 3":
                chatEvent.sender.tell(`\u00A72==========Thieving Unlocks (3/3)==========\n\u00A7aLvl.\u00A7670 \u00A7rPickpocket\u00A79 Librarian\n\u00A7aLvl.\u00A7673 \u00A7rPickpocket\u00A79 Piglin\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7bThieving Skillcape`)
                break;
            case "!ranged":
            case "!rng":
                chatEvent.sender.tell(`\u00A72==========Ranged==========\n\u00A7rRanged is a combat skill trained through fighting enemies with ranged weapons. Leveling up this will grant you the ability to wield more powerful bows and use better arrows. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!ranged unlocks> \u00A7dDisplays Ranged unlock info`)
                break;
            //mining
            case "!mining":
            case "!mine":
                chatEvent.sender.tell(`\u00A72==========Mining==========\n\u00A7rMining is a non-combat skill trained by mining for ore. Leveling up this skill will grant you the ability to wield more sturdier pickaxes. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!mining unlocks (1-2)> \u00A7dDisplays Mining unlock info`)
                break;
            case "!mining unlocks":
            case "!mine unlocks":
            case "!mu":
                chatEvent.sender.tell(`\u00A72==========Mining Unlocks (1/2)==========\n\u00A7aLvl.\u00A761 \u00A7rEquip the \u00A7b Wooden Pickaxe\n\u00A7aLvl.\u00A7610 \u00A7rEquip the \u00A7b Bronze Pickaxe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Iron Pickaxe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Black Pickaxe\n\u00A7aLvl.\u00A7630 \u00A7rEquip the \u00A7b Steel Pickaxe\n\u00A7aLvl.\u00A7640 \u00A7rEquip the \u00A7b Mithril Pickaxe\n\u00A7aLvl.\u00A7650 \u00A7rEquip the \u00A7b Adamant Pickaxe\n\u00A7aLvl.\u00A7660 \u00A7rEquip the \u00A7b Rune Pickaxe`)
                break;
            case "!mining unlocks 2":
            case "!mine unlocks 2":
            case "!mu2":
                chatEvent.sender.tell(`\u00A72==========Mining Unlocks (2/2)==========\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7b Mining Skillcape`)
                break;
            //digging
            case "!digging":
            case "!dig":
                chatEvent.sender.tell(`\u00A72==========Digging==========\n\u00A7rDigging is a non-combat skill trained by digging through loose sediment. Leveling up this skill will grant you the ability to wield more sturdier shovels. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!digging unlocks (1-2)> \u00A7dDisplays Digging unlock info`)
                break;
            case "!digging unlocks":
            case "!dig unlocks":
                chatEvent.sender.tell(`\u00A72==========Digging Unlocks (1/2)==========\n\u00A7aLvl.\u00A761 \u00A7rEquip the \u00A7b Wooden Shovel\n\u00A7aLvl.\u00A7610 \u00A7rEquip the \u00A7b Bronze Shovel\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Iron Shovel\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Black Shovel\n\u00A7aLvl.\u00A7630 \u00A7rEquip the \u00A7b Steel Shovel\n\u00A7aLvl.\u00A7640 \u00A7rEquip the \u00A7b Mithril Shovel\n\u00A7aLvl.\u00A7650 \u00A7rEquip the \u00A7b Adamant Shovel\n\u00A7aLvl.\u00A7660 \u00A7rEquip the \u00A7b Rune Shovel`)
                break;
            case "!digging unlocks 2":
            case "!dig unlocks 2":
                chatEvent.sender.tell(`\u00A72==========Digging Unlocks (2/2)==========\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7b Digging Skillcape`)
                break;
            //farming
            case "!farming":
            case "!farm":
                chatEvent.sender.tell(`\u00A72==========Farming==========\n\u00A7rFarming is a non-combat skill trained by harvesting crops. Leveling up this skill will grant you the ability to wield more sturdier hoes. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!farming unlocks (1-2)> \u00A7dDisplays Farming unlock info`)
                break;
            case "!farming unlocks":
            case "!farm unlocks":
                chatEvent.sender.tell(`\u00A72==========Farming Unlocks (1/2)==========\n\u00A7aLvl.\u00A761 \u00A7rEquip the \u00A7b Wooden Hoe\n\u00A7aLvl.\u00A7610 \u00A7rEquip the \u00A7b Bronze Hoe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Iron Hoe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Black Hoe\n\u00A7aLvl.\u00A7630 \u00A7rEquip the \u00A7b Steel Hoe\n\u00A7aLvl.\u00A7640 \u00A7rEquip the \u00A7b Mithril Hoe\n\u00A7aLvl.\u00A7650 \u00A7rEquip the \u00A7b Adamant Hoe\n\u00A7aLvl.\u00A7660 \u00A7rEquip the \u00A7b Rune Hoe\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7b Farming Skillcape`)
                break;
            case "!farming unlocks 2":
            case "!farm unlocks 2":
                chatEvent.sender.tell(`\u00A72==========Farming Unlocks (2/2)==========\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7b Farming Skillcape`)
                break;
            //woodcutting
            case "!woodcutting":
            case "!cutting":
            case "!wood":
                chatEvent.sender.tell(`\u00A72==========Woodcutting==========\n\u00A7rWoodcutting is a non-combat skill trained by chopping down trees. Leveling up this skill will grant you the ability to wield more sturdier axes. For detailed skill information, check the RuneCraft Wiki.\n\u00A7a<!woodcutting unlocks (1-2)> \u00A7dDisplays Woodcutting unlock info`)
                break;
            case "!woodcutting unlocks":
            case "!cutting unlocks":
            case "!wood unlocks":
            case "!wooodu":
                chatEvent.sender.tell(`\u00A72==========Woodcutting Unlocks (1/2)==========\n\u00A7aLvl.\u00A761 \u00A7rEquip the \u00A7b Wooden Axe\n\u00A7aLvl.\u00A7610 \u00A7rEquip the \u00A7b Bronze Axe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Iron Axe\n\u00A7aLvl.\u00A7620 \u00A7rEquip the \u00A7b Black Axe\n\u00A7aLvl.\u00A7630 \u00A7rEquip the \u00A7b Steel Axe\n\u00A7aLvl.\u00A7640 \u00A7rEquip the \u00A7b Mithril Axe\n\u00A7aLvl.\u00A7650 \u00A7rEquip the \u00A7b Adamant Axe\n\u00A7aLvl.\u00A7660 \u00A7rEquip the \u00A7b Rune Axe`)
                break;
            case "!woodcutting unlocks 2":
            case "!cutting unlocks 2":
            case "!wood unlocks 2":
            case "!wooodu2":
                chatEvent.sender.tell(`\u00A72==========Woodcutting Unlocks (2/2)==========\n\u00A7aLvl.\u00A7699 \u00A7rWear the \u00A7b Woodcutting Skillcape`)
                break;
            //player stat list
            case "!stats":
            case "!s":
                skillPage(chatEvent.sender);
                //\n\uE110d \u00A7dWoodcutting:\u00A73 Lv. §6${getPlayerScore("woodcut_level", chatEvent.sender)} ${getPlayerScore("woodcut_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_woodcut_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("woodcutting_xp_r",chatEvent.sender) - getPlayerScore("woodcutting_xp_c",chatEvent.sender)}`}
                //chatEvent.sender.tell(`\u00A76==========STATS==========\n\uE116 \u00A7dOverall:\u00A73 Lv.§6 ${getPlayerScore("total_level", chatEvent.sender)} ${`§rTotal:`} §a${getPlayerScore("total_xp", chatEvent.sender)} ${`§rXP`}\n\uE108 \u00A7dAgility:\u00A73 Lv. §6${getPlayerScore("agility_level", chatEvent.sender)} ${getPlayerScore("agility_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_agility_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("agility_xp_r", chatEvent.sender) - getPlayerScore("agility_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE114 \u00A7dAttack:\u00A73 Lv. §6${getPlayerScore("attack_level", chatEvent.sender)} ${getPlayerScore("attack_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_attack_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("attack_xp_r", chatEvent.sender) - getPlayerScore("attack_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE112 \u00A7dConstitution:\u00A73 Lv. §6${getPlayerScore("const_level", chatEvent.sender)} ${getPlayerScore("const_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_const_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("const_xp_r", chatEvent.sender) - getPlayerScore("const_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE10a \u00A7dDefense:\u00A73 Lv. §6${getPlayerScore("def_level", chatEvent.sender)} ${getPlayerScore("def_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_def_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("def_xp_r", chatEvent.sender) - getPlayerScore("def_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE10e \u00A7dDigging:\u00A73 Lv. §6${getPlayerScore("digging_level", chatEvent.sender)} ${getPlayerScore("digging_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_digging_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("digging_xp_r", chatEvent.sender) - getPlayerScore("digging_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE10f \u00A7dFarming:\u00A73 Lv. §6${getPlayerScore("farming_level", chatEvent.sender)} ${getPlayerScore("farming_level", chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_farming_xp", chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("farming_xp_r", chatEvent.sender) - getPlayerScore("farming_xp_c", chatEvent.sender)}`} ${`§rXP`}\n\uE10c \u00A7dMining:\u00A73 Lv. §6${getPlayerScore("mining_level", chatEvent.sender)} ${getPlayerScore("mining_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_mining_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("mining_xp_r",chatEvent.sender) - getPlayerScore("mining_xp_c",chatEvent.sender)}`} ${`§rXP`}\n\uE110 \u00A7dPrayer:\u00A73 Lv. §6${getPlayerScore("prayer_level", chatEvent.sender)} ${getPlayerScore("prayer_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_prayer_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("prayer_xp_r",chatEvent.sender) - getPlayerScore("prayer_xp_c",chatEvent.sender)}`} ${`§rXP`}\n\uE115 \u00A7dRanged:\u00A73 Lv. §6${getPlayerScore("ranged_level", chatEvent.sender)} ${getPlayerScore("ranged_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_ranged_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("ranged_xp_r",chatEvent.sender) - getPlayerScore("ranged_xp_c",chatEvent.sender)}`} ${`§rXP`}\n\uE111 \u00A7dThieving:\u00A73 Lv. §6${getPlayerScore("thieving_level", chatEvent.sender)} ${getPlayerScore("thieving_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_thieving_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("thieving_xp_r",chatEvent.sender) - getPlayerScore("thieving_xp_c",chatEvent.sender)}`} ${`§rXP`}\n\uE10d \u00A7dWoodcutting:\u00A73 Lv. §6${getPlayerScore("woodcut_level", chatEvent.sender)} ${getPlayerScore("woodcut_level",chatEvent.sender) == 120 ? `§rTotal: §a${getPlayerScore("tot_woodcut_xp",chatEvent.sender)}` : `§rTo Next: §a${getPlayerScore("woodcutting_xp_r",chatEvent.sender) - getPlayerScore("woodcutting_xp_c",chatEvent.sender)}`} ${`§rXP`}`)
                break;
            //discord server
            case "!teams":
            case "!team":
                chatEvent.sender.tell(`\u00A72==========TEAMS INFO==========\n\u00A7rJoining a team will prevent receiving damage from other players in the same team. Players can opt to not join a team, or be a member of only one team. Players can switch and leave teams at any time.\n\u00A7a<!team join (1-4)> \u00A7dJoins the specified team\n\u00A7a<!team leave> \u00A7dLeave the curent team you are a part of`)
                break;
            case "!team join 1":
            case "!tj1":
            case "!t join 1":
                if (chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.removeTag("team4");
                    chatEvent.sender.tell(`\u00A7aYou have left team 4.`)
                };
                if (chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.removeTag("team3");
                    chatEvent.sender.tell(`\u00A7aYou have left team 3.`)
                };
                if (chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.removeTag("team2");
                    chatEvent.sender.tell(`\u00A7aYou have left team 2.`)
                };
                if (chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.tell(`\u00A7aYou are already in team 1.`)
                };
                if (!chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.addTag("team1");
                    chatEvent.sender.tell(`\u00A7aYou have joined team 1.`)
                };
                break;
            case "!team join 2":
            case "!tj2":
            case "!t join 2":
                if (chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.removeTag("team4");
                    chatEvent.sender.tell(`\u00A7aYou have left team 4.`)
                };
                if (chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.removeTag("team3");
                    chatEvent.sender.tell(`\u00A7aYou have left team 3.`)
                };
                if (chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.removeTag("team1");
                    chatEvent.sender.tell(`\u00A7aYou have left team 1.`)
                };
                if (chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.tell(`\u00A7aYou are already in team 2.`)
                };
                if (!chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.addTag("team2");
                    chatEvent.sender.tell(`\u00A7aYou have joined team 2.`)
                };
                break;
            case "!team join 3":
            case "!tj3":
            case "!t join 3":
                if (chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.removeTag("team4");
                    chatEvent.sender.tell(`\u00A7aYou have left team 4.`)
                };
                if (chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.removeTag("team1");
                    chatEvent.sender.tell(`\u00A7aYou have left team 1.`)
                };
                if (chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.removeTag("team2");
                    chatEvent.sender.tell(`\u00A7aYou have left team 2.`)
                };
                if (chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.tell(`\u00A7aYou are already in team 3.`)
                };
                if (!chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.addTag("team3");
                    chatEvent.sender.tell(`\u00A7aYou have joined team 3.`)
                };
                break;
            case "!team join 4":
            case "!tj4":
            case "!t join 4":
                if (chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.removeTag("team1");
                    chatEvent.sender.tell(`\u00A7aYou have left team 1.`)
                };
                if (chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.removeTag("team3");
                    chatEvent.sender.tell(`\u00A7aYou have left team 3.`)
                };
                if (chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.removeTag("team2");
                    chatEvent.sender.tell(`\u00A7aYou have left team 2.`)
                };
                if (chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.tell(`\u00A7aYou are already in team 4.`)
                };
                if (!chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.addTag("team4");
                    chatEvent.sender.tell(`\u00A7aYou have joined team 4.`)
                };
                break;
            case "!team leave":
            case "!tl":
            case "!t leave":
                if (chatEvent.sender.hasTag("team1")) {
                    chatEvent.sender.removeTag("team1");
                    chatEvent.sender.tell(`\u00A7aYou have left team 1.`)
                };
                if (chatEvent.sender.hasTag("team3")) {
                    chatEvent.sender.removeTag("team3");
                    chatEvent.sender.tell(`\u00A7aYou have left team 3.`)
                };
                if (chatEvent.sender.hasTag("team2")) {
                    chatEvent.sender.removeTag("team2");
                    chatEvent.sender.tell(`\u00A7aYou have left team 2.`)
                };
                if (chatEvent.sender.hasTag("team4")) {
                    chatEvent.sender.removeTag("team4");
                    chatEvent.sender.tell(`\u00A7aYou have left team 4.`)
                };
                break;
            case "!wiki":
                chatEvent.sender.tell(`\u00A72==========RuneCraft Wiki==========\n\u00A7rThe RuneCraft Wiki is a database with detailed information on every skill, item, and element of RuneCraft. To access to the Wiki, please enter the following link into your browser: \u00A79https://runecraft.fandom.com/wiki/RuneCraft_Wiki`)
                break;
            case "!youtube":
                chatEvent.sender.tell(`\u00A72==========Youtube==========\n\u00A7rMy Youtube channel is where I make development previews, guides, and general fun videos. If you want to support this add-on, subscribing and joining the community is the best route to doing so: \u00A79https://www.youtube.com/channel/UCZnU2d_P45hy5LHbCQ3xiOA`)
                break;
            case "!disc":
            case "!discord":
                chatEvent.sender.tell(`\u00A72==========RuneCraft Discord==========\n\u00A7rThe RuneCraft Discord server is a place to discuss RuneCraft, get help, and provide feedback/vote on new mechanics for the add-on: \u00A79https://discord.gg/tTfKQEJJ5d`)
                break;
            //player enters in an incorrect command
            default: chatEvent.sender.tell(`\u00A7cTrying to find a command? Type !help for a list of all commands.`);
        }
    }
    else {
        //${getPlayerScore("attack_level", player)} ${getPlayerScore("attack_level", player) == 120 ? ${getPlayerScore("tot_attack_xp", player)}
        let attackLevel = getPlayerScore("attack_level", chatEvent.sender) < 60 ? `${getPlayerScore("attack_level", chatEvent.sender)}`:`${60}`
        let defenseLevel = getPlayerScore("def_level", chatEvent.sender) < 60 ? `${getPlayerScore("def_level", chatEvent.sender)}`:`${60}`
        let constitutionLevel = getPlayerScore("const_level", chatEvent.sender) < 99 ? `${getPlayerScore("const_level", chatEvent.sender)}`:`${99}`
        let prayerLevel = getPlayerScore("prayer_level", chatEvent.sender) < 73 ? `${getPlayerScore("prayer_level", chatEvent.sender)}`:`${73}`

        let combatLevel = Math.round(1.3 * (parseInt(attackLevel) + parseInt(defenseLevel) + parseInt(constitutionLevel) + (parseInt(prayerLevel)/2))/4)
        world.say(`<${chatEvent.sender.name}> [§6${combatLevel}§r] ${message}`)
        chatEvent.cancel = true;
    }
})

world.events.beforeItemUse.subscribe((eventData) => {
    if (eventData.item.typeId == "runecraft:stat_tracker") {
        skillPage(eventData.source);
    }
    return;
})

async function skillPage(player) {
    player.tell(`\u00A76==========STATS==========\n\uE116 \u00A7dOverall:\u00A73 Lv.§6 ${getPlayerScore("total_level", player)} ${`§rTotal:`} §a${getPlayerScore("total_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')} §rXP
\uE108 \u00A7dAgility:\u00A73 Lv. §6${getPlayerScore("agility_level", player)} ${getPlayerScore("agility_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_agility_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("agility_xp_r", player) - getPlayerScore("agility_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE114 \u00A7dAttack:\u00A73 Lv. §6${getPlayerScore("attack_level", player)} ${getPlayerScore("attack_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_attack_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("attack_xp_r", player) - getPlayerScore("attack_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE112 \u00A7dConstitution:\u00A73 Lv. §6${getPlayerScore("const_level", player)} ${getPlayerScore("const_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_const_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("const_xp_r", player) - getPlayerScore("const_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE10a \u00A7dDefense:\u00A73 Lv. §6${getPlayerScore("def_level", player)} ${getPlayerScore("def_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_def_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("def_xp_r", player) - getPlayerScore("def_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE10e \u00A7dDigging:\u00A73 Lv. §6${getPlayerScore("digging_level", player)} ${getPlayerScore("digging_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_digging_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("digging_xp_r", player) - getPlayerScore("digging_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE10f \u00A7dFarming:\u00A73 Lv. §6${getPlayerScore("farming_level", player)} ${getPlayerScore("farming_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_farming_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("farming_xp_r", player) - getPlayerScore("farming_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE10c \u00A7dMining:\u00A73 Lv. §6${getPlayerScore("mining_level", player)} ${getPlayerScore("mining_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_mining_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("mining_xp_r", player) - getPlayerScore("mining_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE110 \u00A7dPrayer:\u00A73 Lv. §6${getPlayerScore("prayer_level", player)} ${getPlayerScore("prayer_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_prayer_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("prayer_xp_r", player) - getPlayerScore("prayer_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE115 \u00A7dRanged:\u00A73 Lv. §6${getPlayerScore("ranged_level", player)} ${getPlayerScore("ranged_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_ranged_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("ranged_xp_r", player) - getPlayerScore("ranged_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE111 \u00A7dThieving:\u00A73 Lv. §6${getPlayerScore("thieving_level", player)} ${getPlayerScore("thieving_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_thieving_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("thieving_xp_r", player) - getPlayerScore("thieving_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP
\uE10d \u00A7dWoodcutting:\u00A73 Lv. §6${getPlayerScore("woodcut_level", player)} ${getPlayerScore("woodcut_level", player) == 120 ? `§rTotal:§a ${getPlayerScore("tot_woodcut_xp", player).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}` : `§rTo Next: §a${(getPlayerScore("woodcutting_xp_r", player) - getPlayerScore("woodcutting_xp_c", player)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ',')}`} §rXP`)
    return;
}

//skill master interact
system.events.scriptEventReceive.subscribe((eventData) => {
    if (eventData.id == "runecraft:skill_master") {
        skillcapeShop(eventData.sourceEntity);
    }
})

const skillLevelScoreboards = [
    "agility_level",
    "attack_level",
    "const_level",
    "def_level",
    "digging_level",
    "farming_level",
    "mining_level",
    "prayer_level",
    "ranged_level",
    "thieving_level",
    "woodcut_level",
]

const skillcapeNames = [
    "\u00A75Agility Skillcape",
    "\u00A75Attack Skillcape",
    "\u00A75Constitution Skillcape",
    "\u00A75Defense Skillcape",
    "\u00A75Digging Skillcape",
    "\u00A75Farming Skillcape",
    "\u00A75Mining Skillcape",
    "\u00A75Prayer Skillcape",
    "\u00A75Ranged Skillcape",
    "\u00A75Thieving Skillcape",
    "\u00A75Woodcutting Skillcape"
]

const itemIdentifiers = [
    "runecraft:agility_skillcape",
    "runecraft:attack_skillcape",
    "runecraft:constitution_skillcape",
    "runecraft:defense_skillcape",
    "runecraft:digging_skillcape",
    "runecraft:farming_skillcape",
    "runecraft:mining_skillcape",
    "runecraft:prayer_skillcape",
    "runecraft:ranged_skillcape",
    "runecraft:thieving_skillcape",
    "runecraft:woodcutting_skillcape",
]

async function skillcapeShop(player) {
    let skillLevelArray = [
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0,
        0
    ]

    const textureLocations = [
        "textures/items/skillcapes/agi.png",
        "textures/items/skillcapes/ati.png",
        "textures/items/skillcapes/coi.png",
        "textures/items/skillcapes/dei.png",
        "textures/items/skillcapes/dii.png",
        "textures/items/skillcapes/fai.png",
        "textures/items/skillcapes/mii.png",
        "textures/items/skillcapes/pri.png",
        "textures/items/skillcapes/rai.png",
        "textures/items/skillcapes/thi.png",
        "textures/items/skillcapes/woi.png"
    ]
    let isMastery = 0
    const form = new ActionFormData()
        .title("§bSkillcape Shop")

    for (let i = 0; i < skillLevelArray.length; i++) {
        skillLevelArray[i] = getPlayerScore(skillLevelScoreboards[i], player)
        if (skillLevelArray[i] > 98) {
            isMastery = 1
        }
    }

    if (isMastery == 0) {
        form.body("You can only purchase Skillcapes if you have reached Mastery (Level §a99§r or higher) with a Skill")
        form.button("Exit")
        const response = await form.show(player)
    }
    else {
        form.body("Select a Skillcape to purchase.\nAll Skillcapes have a set cost of §a64 Emeralds§r.\n")
        for (let i = 0; i < skillLevelScoreboards.length; i++) {
            if (skillLevelArray[i] > 98) {
                form.button(`${skillcapeNames[i]}`, `${textureLocations[i]}`)
            }
        }

        let emeraldCount = 0;
        const response = await form.show(player)
        if (response.selection != null) {

            //read the players inventory for a stack of emeralds
            //if I detect a stack of emeralds, then I will take a stack of emeralds and give them the skillcape. If I do not detect a stack of emeralds, I will output an error message.
            const playerInventory = player.getComponent("minecraft:inventory").container
            for (let i = 0; i < playerInventory.size && emeraldCount < 64; i++) {
                //get current slot item
                let currentItem = playerInventory.getItem(i)
                if (currentItem) {
                    //check if current slot contains emeralds
                    if (currentItem.typeId == "minecraft:emerald") {

                        //adds the amount of emeralds to emeraldCount
                        emeraldCount = emeraldCount + currentItem.amount;
                        if (emeraldCount > 64) {
                            emeraldCount = 64
                        }
                    }
                }
            };
            //TWO CASES
            //CASE 1: Player has >= 64 emeralds
            //CASE 2: Player has < 64 emeralds
            if (emeraldCount >= 64) {
                for (let i = 0; i < playerInventory.size && emeraldCount != 0; i++) {
                    let currentItem = playerInventory.getItem(i)

                    if (currentItem) {

                        //TWO CASES
                        //CASE 1: 15-10 = emeraldCount of 5
                        //CASE 2: 15-20 = emeraldCount of -5, need to set subtractAmount to emeraldCount instead
                        if (currentItem.typeId == "minecraft:emerald") {
                            let subtractAmount = currentItem.amount;
                            if (subtractAmount > emeraldCount) {
                                let remainingItems = new ItemStack(Items.get(`minecraft:emerald`), (subtractAmount - emeraldCount));
                                subtractAmount = emeraldCount;
                                playerInventory.setItem(i, remainingItems)
                            }
                            else {
                                playerInventory.clearItem(i)
                            }
                            emeraldCount = emeraldCount - subtractAmount
                        }
                    }
                }
                let skillcape = 0;
                for (let i = 0; i < skillLevelArray.length; i++) {
                    let skill = skillLevelArray[i]
                    if (skill >= 99) {
                        if (response.selection == skillcape) {
                            let giveItem = new ItemStack(Items.get(`${itemIdentifiers[i]}`), 1)
                            playerInventory.addItem(giveItem)
                            break;
                        }
                        skillcape = skillcape + 1;
                    }
                }
            }
            else {
                player.tell(`§o§eRequired to purchase skillcape: 60 Emeralds.`);
            }
        }
    }
}

const playerProperties = new DynamicPropertiesDefinition();
playerProperties.defineNumber('fallDamage')

world.events.worldInitialize.subscribe((eventData) => {
    eventData.propertyRegistry.registerEntityTypeDynamicProperties(playerProperties, MinecraftEntityTypes.player);
})

world.events.entityHurt.subscribe((eventData) => {
    if(eventData.damageSource.cause == "fall" && eventData.hurtEntity.typeId == "minecraft:player") {
        eventData.hurtEntity.setDynamicProperty('fallDamage', (eventData.damage * 0.6) + 1)
    }
})

system.events.scriptEventReceive.subscribe((eventData) => {
    if(eventData.id == "runecraft:agility_system") {
        let player = eventData.sourceEntity;
        //player.tell(` PLAYER VELOCITY: ${player.velocity.y}`);
        if (player.velocity.y < 0 && getPlayerScore("gamemode",player) <= 2 && player.getEffect(MinecraftEffectTypes.slowFalling) == undefined && player.velocity.y != -0.12000274658203125 && player.velocity.y != -0.1999969482421875 && player.velocity.y != -0.0039215087890625 && player.hasTag("noAgilityXP") == false && player.getComponent("minecraft:health").current > 0) {
            if(getPlayerScore("gamemode", player) <= 2) {
                if(getPlayerScore("falling_timer",player) < 6) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 1`)                
                }
                else if(getPlayerScore("falling_timer",player) >= 6 && getPlayerScore("falling_timer",player) < 24) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 10`)                
                }
                else if(getPlayerScore("falling_timer", player) >= 24 && getPlayerScore("falling_timer", player) < 30) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 20`)                
                }
                else if(getPlayerScore("falling_timer", player) >= 30 && getPlayerScore("falling_timer", player) < 70) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 4`)
                }
                else if(getPlayerScore("falling_timer", player) >= 70 && getPlayerScore("falling_timer", player) < 120) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 5`)  
                }
                else if(getPlayerScore("falling_timer", player) >= 120 && getPlayerScore("falling_timer", player) < 195) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 6`)
                }
                else if(getPlayerScore("falling_timer", player) >= 195 && getPlayerScore("falling_timer", player) < 295) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 7`)
                }
                else if(getPlayerScore("falling_timer", player) >= 295) {
                    player.runCommandAsync(`scoreboard players add @s falling_timer 1`)
                }
            }
        }
        else if (player.hasTag("noAgilityXP") == true || player.getComponent("minecraft:health").current <= 0 || player.velocity.y >= 0.8 || player.velocity.y == -0.12000274658203125 || player.velocity.y == -0.1999969482421875 || player.velocity.y == -0.0039215087890625) {
            player.runCommandAsync("scoreboard players set @s falling_timer 0");
        }
        else if (player.velocity.y < 0.8 && player.velocity.y >= 0 && getPlayerScore('falling_timer', player) > 0) {
            let agilityXp = getPlayerScore('falling_timer', player);
            agilityXp = Math.floor((agilityXp / 4) * player.getDynamicProperty('fallDamage')); //player.getDynamicProperty('fallDamage')
            player.setDynamicProperty('fallDamage', 1);
            if (getPlayerScore("gamemode", player) <= 2) {
                player.runCommandAsync(`scoreboard players add @s agility_xp_c ${agilityXp}`)
                player.runCommandAsync(`scoreboard players add @s tot_agility_xp ${agilityXp}`)
                player.runCommandAsync(`scoreboard players add @s total_xp ${agilityXp}`)
                player.runCommandAsync(`scoreboard players set @s falling_timer 0`)
            }
        }
    }
})